import 'package:aibiblegames/pages/HomeScreen.dart';
import 'package:aibiblegames/pages/SplashScreen.dart';
import 'package:aibiblegames/utils/CommonUtils.dart';
import 'package:cloud_functions/cloud_functions.dart';
import 'package:dart_openai/openai.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:flutter_stripe/flutter_stripe.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:loader_overlay/loader_overlay.dart';
import 'package:flutter_native_splash/flutter_native_splash.dart';
import 'package:firebase_core/firebase_core.dart';
import 'firebase_options.dart';
final GlobalKey<NavigatorState> _navigator = GlobalKey<NavigatorState>();
Future main() async{
  await dotenv.load(fileName: ".env", mergeWith: {
    'TEST_VAR': '5',
  });
  WidgetsBinding widgetsBinding = WidgetsFlutterBinding.ensureInitialized();
  MobileAds.instance.initialize();
  OpenAI.apiKey = dotenv.env['CHATGPT_API_KEY']!;
  FlutterNativeSplash.preserve(widgetsBinding: widgetsBinding);
  await Firebase.initializeApp(
    options: DefaultFirebaseOptions.currentPlatform,
  );
  Stripe.publishableKey = dotenv.env['STRIPE_PUBLIC']!;
  runApp(const MyHomePage(title: "AI Bible games"));
  FlutterNativeSplash.remove();
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key, required this.title});
  final String title;
  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  int _counter = 0;
  Widget screenView = Container( child: Text("Welcome to AI Bible Games"),);
  final GlobalKey<ScaffoldMessengerState> rootScaffoldMessengerKey = GlobalKey<ScaffoldMessengerState>();
  @override
  void initState() {
    super.initState();
    checkAuthenticate();
  }
  void checkAuthenticate(){

      if (FirebaseAuth.instance.currentUser != null && FirebaseAuth.instance.currentUser?.emailVerified == true) {

        setState(() {
          screenView = HomeScreen();
        });
      } else {
        setState(() {
          screenView = SplashScreen();
        });
        // _navigator.currentState!.pushReplacementNamed('/home');
      }


  }
  @override
  Widget build(BuildContext context) {
    return GlobalLoaderOverlay(child: MaterialApp(
      title: 'AI Bible Game',
      scaffoldMessengerKey: rootScaffoldMessengerKey,
      theme: ThemeData(
          primarySwatch: Colors.indigo,
          visualDensity: VisualDensity.adaptivePlatformDensity
      ),
      navigatorKey: _navigator,
      home: Scaffold(
        resizeToAvoidBottomInset: true,
        body: Center(
          child: screenView,
        ),
      ),
      routes: CommonUtils.returnRoutes(context),
    ), overlayColor: Colors.black45,);
  }
}
