import 'dart:convert';
import 'dart:io' show Platform;
import 'dart:math';

import 'package:aibiblegames/utils/CommonUtils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_chat_ui/flutter_chat_ui.dart';
import 'package:flutter_chat_types/flutter_chat_types.dart' as types;
import 'package:dart_openai/openai.dart';
import 'package:flutter/services.dart' show rootBundle;
import 'package:flutter_dotenv/flutter_dotenv.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:loader_overlay/loader_overlay.dart';
import 'package:printing/printing.dart';
import 'package:pdf/pdf.dart';

class ChatScreen extends StatefulWidget{
  final String gameMode;
  final String userName;
  String? gameTitle;
  String? promptUrl;
  ChatScreen({super.key, required this.gameMode, required this.userName, this.gameTitle, this.promptUrl});
  @override
  State<ChatScreen> createState() => _ChatScreenState();
}
class _ChatScreenState extends State<ChatScreen>{
  final List<types.Message> _messages = [];
  final List<OpenAIChatCompletionChoiceMessageModel> _aiMessages = [];
  late types.User ai;
  late types.User user;
  var chatResponseId = '';
  var chatResponseContent = '';
  bool isAiTyping = false;

  RewardedAd? _rewardedAd;
  int _numRewardedLoadAttempts = 0;

  RewardedInterstitialAd? _rewardedInterstitialAd;
  int _numRewardedInterstitialLoadAttempts = 0;
  int _turnsCount = 0;
  @override
  void initState() {
    super.initState();
    ai = const types.User(id: 'ai', firstName: 'AI');
    user = const types.User(id: 'user', firstName: 'You');
    if(widget.gameMode == "free") {
      _createRewardedAd();
    }
    else{
      initChatPrompt();
    }

  }
  void initChatPrompt() async{

    context.loaderOverlay.show();
    try{
      String preDefinedPrompt = "";

      String? promptUrl = widget.promptUrl ?? dotenv.env['FREE_PROMPT_URL'];
      if (widget.gameMode == "free"){

        if(promptUrl == null){
          CommonUtils.showToast("Error while init game.");
          Navigator.pop(context);
        }
      }

      preDefinedPrompt = await CommonUtils.fetchData(promptUrl!) ;

      context.loaderOverlay.hide();
      if(preDefinedPrompt != "") {
        preDefinedPrompt = preDefinedPrompt.replaceAll("<\$user_name>", widget.userName);
        preDefinedPrompt = preDefinedPrompt.replaceAll("\$user_name", widget.userName);
        preDefinedPrompt = preDefinedPrompt.replaceAll("user_name", widget.userName);
        _aiMessages.add(OpenAIChatCompletionChoiceMessageModel(
            content: preDefinedPrompt,
            role: OpenAIChatMessageRole.system
        ));
        listenStreaming();
      }
      else{
        CommonUtils.showToast("Error while init game");
        Navigator.pop(context);
      }
    }
    catch(e){
      context.loaderOverlay.hide();
      CommonUtils.showToast(e.toString());
      Navigator.pop(context);
    }

  }
  String randomString() {
    final random = Random.secure();
    final values = List<int>.generate(16, (i) => random.nextInt(255));
    return base64UrlEncode(values);
  }
  void _handleSendPressed(types.PartialText message) async {
    final textMessage = types.TextMessage(
      author: user,
      createdAt: DateTime.now().millisecondsSinceEpoch,
      id: randomString(),
      text: message.text,
    );

    _addMessage(textMessage);
    // _saveMessage(message.text, MessageRole.user);
    _completeChat(message.text);
  }
  // add new bubble to chat
  void _addMessage(types.Message message) {
    setState(() {
      _messages.insert(0, message);
    });
  }
  void _addImage(String url) async{
    int? imageSize = await CommonUtils.getFileSizeFromUrl(url);
    if(imageSize != null){
      print("Image Size: ${imageSize}");
      final message = types.ImageMessage(
          name: "image",
          author: ai,
          width: 100, height: 100,
          createdAt: DateTime.now().millisecondsSinceEpoch,
          id: randomString(),
          uri: url,

          size: imageSize!
      );
      print("Add Image Message");
      _addMessage(message);
    }

  }

  // modify last bubble in chat
  void _addMessageStream(String message) {
    setState(() {
      _messages.first =
          (_messages.first as types.TextMessage).copyWith(text: message);
    });
  }
  void _completeChat(String prompt) async {
    _aiMessages.add(OpenAIChatCompletionChoiceMessageModel(
      content: prompt,
      role: OpenAIChatMessageRole.user,
    ));
    listenStreaming();
  }
  List<String> extractImageUrls(String input) {
    // Regular expression to match URLs in various formats
    RegExp urlRegex = RegExp(r'https?://[\w\-\.]+[\w]{2,}(?:/[^/#?]+)+\.(?:jpg|jpeg|png|gif)', caseSensitive: false);

    RegExp imgRegex = RegExp(r'!\[.*?\]\((.*?)\)|\[img=.*?\](.*?)\[/img\]|^https?://.*?\.(jpg|jpeg|png|gif)$', multiLine: true);

    var matches = imgRegex.allMatches(input);

    // List to store extracted image URLs
    List<String> imageUrls = [];

    for (var match in matches) {
      if (urlRegex.hasMatch(match.group(0)!)) {
        String? url = match.group(1) ?? match.group(2) ?? match.group(0);
        if (url != null) {
          imageUrls.add(url);
        }
      }
    }

    return imageUrls;
  }

  String removeLinesWithImageUrls(String input) {
    // Extract image URLs from the input string
    List<String> imageUrls = extractImageUrls(input);

    // Split the input string into lines and remove lines containing the image URLs
    List<String> lines = input.split('\n').where((line) => !imageUrls.any((url) => line.contains(url))).toList();

    // Join the lines back into a single string and return
    return lines.join('\n');
  }

  void listenStreaming(){

    Stream<OpenAIStreamChatCompletionModel> chatStream =
    OpenAI.instance.chat.createStream(
      model: "gpt-3.5-turbo-16k",
      messages: _aiMessages,
      temperature: 0.7,
      n: 1,
      frequencyPenalty: 0,
      presencePenalty: 0
    );

    chatStream.listen((chatStreamEvent) {
      print("$chatResponseId: ${chatStreamEvent.id}");
      // existing id: just update to the same text bubble
      if (chatResponseId == chatStreamEvent.id) {
        chatResponseContent +=
            chatStreamEvent.choices.first.delta.content ?? '';

        _addMessageStream(chatResponseContent);

        if (chatStreamEvent.choices.first.finishReason == "stop") {

          isAiTyping = false;
          _aiMessages.add(OpenAIChatCompletionChoiceMessageModel(
            content: chatResponseContent,
            role: OpenAIChatMessageRole.assistant,
          ));

          List<String> extractedUrls = extractImageUrls(chatResponseContent);
          for (var imageUrl in extractedUrls) {
            if(imageUrl != null){
              _addImage(imageUrl!);
            }

          }
          chatResponseContent = removeLinesWithImageUrls(chatResponseContent);
          _addMessageStream(chatResponseContent);
          // _saveMessage(chatResponseContent, MessageRole.ai);
          chatResponseId = '';
          chatResponseContent = '';
          _turnsCount ++ ;
          if(_turnsCount == 1 || _turnsCount % 5 == 0){
            if(widget.gameMode == "free"){
              _showRewardedAd();
            }
          }
        }
      } else {
        // new id: create new text bubble
        chatResponseId = chatStreamEvent.id;
        chatResponseContent = chatStreamEvent.choices.first.delta.content ?? '';
        onMessageReceived(id: chatResponseId, message: chatResponseContent);
        isAiTyping = true;
      }
    });
  }
  void _showRewardedAd() {
    if (_rewardedAd == null) {
      print('Warning: attempt to show rewarded before loaded.');
      _createRewardedAd();
      return;
    }
    _rewardedAd!.fullScreenContentCallback = FullScreenContentCallback(
      onAdShowedFullScreenContent: (RewardedAd ad) =>
          print('ad onAdShowedFullScreenContent.'),
      onAdDismissedFullScreenContent: (RewardedAd ad) {
        print('$ad onAdDismissedFullScreenContent.');
        ad.dispose();
        _createRewardedAd();
      },
      onAdFailedToShowFullScreenContent: (RewardedAd ad, AdError error) {
        print('$ad onAdFailedToShowFullScreenContent: $error');
        ad.dispose();
        _createRewardedAd();
      },
    );

    _rewardedAd!.setImmersiveMode(true);
    _rewardedAd!.show(
        onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
          print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
        });
    _rewardedAd = null;
  }

  void onMessageReceived({String? id, required String message}) {
    print("ON Message Received");
    var newMessage = types.TextMessage(
      author: ai,
      id: id ?? DateTime.now().millisecondsSinceEpoch.toString(),
      text: message,
      createdAt: DateTime.now().millisecondsSinceEpoch,
    );
    _addMessage(newMessage);
  }
  static final AdRequest request = AdRequest(
    keywords: <String>['foo', 'bar'],
    contentUrl: 'http://foo.com/bar.html',
    nonPersonalizedAds: true,
  );


  int maxFailedLoadAttempts = 3;
  void _createRewardedAd() {
    context.loaderOverlay.show();
    RewardedAd.load(
        adUnitId: Platform.isAndroid
            ? 'ca-app-pub-8839366480674347/7829176172'
            : 'ca-app-pub-8839366480674347/6324522811',
        request: const AdRequest(),
        rewardedAdLoadCallback: RewardedAdLoadCallback(
          onAdLoaded: (RewardedAd ad) {
            print('$ad loaded.');
            _rewardedAd = ad;
            _numRewardedLoadAttempts = 0;
            context.loaderOverlay.hide();
            if(_aiMessages.isEmpty){
              initChatPrompt();
            }

          },
          onAdFailedToLoad: (LoadAdError error) {
            print('RewardedAd failed to load: $error');



            _rewardedAd = null;
            _numRewardedLoadAttempts += 1;
            if (_numRewardedLoadAttempts < maxFailedLoadAttempts) {
              _createRewardedAd();
            }
            else{
              CommonUtils.showToast("Error while loading the game.");
              Navigator.pop(context);
              context.loaderOverlay.hide();
            }
          },
        ));
  }
  String applyLineBreaks(String html) {
    // Replace \n with <br> tags in the HTML string
    return html.replaceAll('\n', '<br>');
  }
  String convertMarkdownToHTML(String markdownText) {
    // Regular expression to match Markdown image syntax ![alt text](image_url)
    final markdownImageRegex = RegExp(r'\!\[([^\]]*)\]\(([^\)]+)\)');

    // Regular expression to match [img=widthxheight]image_url[/img] format
    final imgTagRegex = RegExp(r'\[img=([0-9]+)x([0-9]+)\]([^\[]+)\[/img\]');

    // Replace Markdown image syntax with HTML image tags
    String htmlText = markdownText.replaceAllMapped(markdownImageRegex, (match) {
      final altText = match.group(1);
      final imageUrl = match.group(2);
      return '<img src="$imageUrl" alt="$altText" width="300" height="300" />';
    });

    // Replace [img] tags with HTML image tags
    htmlText = htmlText.replaceAllMapped(imgTagRegex, (match) {
      final width = match.group(1);
      final height = match.group(2);
      final imageUrl = match.group(3);
      return '<img src="$imageUrl" width="$width" height="$height" />';
    });

    return htmlText;
  }


  @override
  Widget build(BuildContext context){
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.gameTitle ?? 'Welcome to the AI Bible Games'),
        actions: [
          InkWell(
            onTap: ()async{
              print('Print');
              String htmlStr = '';
              _aiMessages.forEach((element) {

                if(element.role != OpenAIChatMessageRole.system){
                 htmlStr += '<p>${element.role == OpenAIChatMessageRole.user ? 'You' : 'AI'}: ${element.content}</p>';
                }
              });
              htmlStr = convertMarkdownToHTML(htmlStr);
              htmlStr = applyLineBreaks(htmlStr);
              await Printing.layoutPdf(
                  onLayout: (PdfPageFormat format) async => await Printing.convertHtml(
                    format: format,
                    html: '<html><body>${htmlStr}</body></html>',
                  ));
            },
            child: Container(
              padding: EdgeInsets.all(10),
              child: Icon(Icons.picture_as_pdf),
            ),
          )
        ],
      ),
      body: Chat(
          messages: _messages, onSendPressed: _handleSendPressed, user: user
      ),
    );
  }
}