import 'package:aibiblegames/utils/MyColors.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:url_launcher/url_launcher.dart';

import '../utils/MyStyles.dart';
import 'ChatScreen.dart';
import 'SubscriptionsScreen.dart';
FirebaseFirestore firestore = FirebaseFirestore.instance;
class HomeScreen extends StatefulWidget{
  @override
  State<HomeScreen> createState() => _HomeScreenState();

}

class _HomeScreenState extends State<HomeScreen>{
  String userName = '';
  String subscriptionPlan = 'Free';
  final Stream<DocumentSnapshot> documentStream = firestore.collection('users').doc(FirebaseAuth.instance.currentUser?.uid).snapshots();
  @override
  void initState() {
    super.initState();

    FirebaseAuth.instance
        .authStateChanges()
        .listen((User? user) async{
      if (user == null) {
        print('User is currently signed out!');
        Navigator.pushReplacementNamed(context, '/splash');
      } else {

        setState(() {
          userName = FirebaseAuth.instance.currentUser?.displayName ?? '';
        });
      }
    });


  }
  String extractInitials(String name) {
    List<String> words = name.split(" ");
    String initials = "";

    for (String word in words) {
      if (word.isNotEmpty) {
        initials += word[0].toUpperCase();
      }
    }

    return initials;
  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(

      body: StreamBuilder<DocumentSnapshot> (
        stream: documentStream,
        builder: (context, snapshot){
          dynamic activeSubscription = snapshot.data?.data().toString().contains('subscription') == true ? snapshot.data!.get('subscription') : null;
          bool hasSubscription = activeSubscription != null && activeSubscription['status'] == 'active';
          return  SafeArea(
            child: Container(
              color: MyColors.backgroundColor,
              height: double.infinity,
              child: Column(
                children: [
                  Expanded(child: SingleChildScrollView(
                    child: Column(
                      children: [
                        SizedBox(
                          height: 350,

                          child: Stack(
                            children: [
                              Center(

                                child: Image.asset("assets/blogo.png"),
                              ),
                              Container(
                                alignment: Alignment.bottomCenter,
                                child: Text(
                                  "AI BIBLE\nGAMES".toUpperCase(),
                                  style: const TextStyle(
                                      fontSize: 70, color: Colors.white, fontFamily: 'Jomhuria', height: 0.5),
                                  textAlign: TextAlign.center,
                                ),
                              )
                            ],
                          ),
                        ),
                        SizedBox(height: 30,),
                        Center(child: GestureDetector(
                          onTap: () async{
                            if(hasSubscription){
                              Navigator.of(context).pushNamed('/games');
                            }
                            else{

                              var result = await Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (_) => SubscriptionsScreen(
                                      fromScreen: 'home'
                                  ),
                                ),
                              );
                              if(result == 0){
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (_) => ChatScreen(gameMode: 'free',userName: userName,),
                                  ),
                                );
                              }
                              else if (result == 1){
                                Navigator.of(context).pushNamed('/games');
                              }
                            }

                          },
                          child: Stack(
                            children: [
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Image.asset(
                                  'assets/bg_btn.png',
                                  fit: BoxFit.contain,
                                ),
                              ),
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Center(
                                  child: Text(hasSubscription ? 'NEW GAME' : 'FREE GAME', style: MyStyles.primaryButtonStyle,),
                                ),
                              )
                            ],
                          ),
                        ),),
                        SizedBox(height: 10,),
                        Center(child: GestureDetector(
                          onTap: () async{

                            Navigator.of(context).pushNamed('/subscriptions');
                          },
                          child: Stack(
                            children: [
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Image.asset(
                                  hasSubscription ? 'assets/bg_btn.png' : 'assets/bg_btn_inactive.png' ,
                                  fit: BoxFit.contain,
                                ),
                              ),
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Center(
                                  child: Text(hasSubscription ? 'ACCOUNT' : 'SUBSCRIPTION', style: MyStyles.primaryButtonStyle,),
                                ),
                              )
                            ],
                          ),
                        ),),
                        SizedBox(height: 10,),
                        Center(child: GestureDetector(
                          onTap: () async{
                            FirebaseAuth.instance.signOut();
                          },
                          child: Stack(
                            children: [
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Image.asset(
                                  'assets/bg_btn.png',
                                  fit: BoxFit.contain,
                                ),
                              ),
                              SizedBox(
                                width: 200,
                                height: 50,
                                child: Center(
                                  child: Text('LOGOUT', style: MyStyles.primaryButtonStyle,),
                                ),
                              )
                            ],
                          ),
                        ),),

                      ],
                    ),
                  )),
                  Row(
                    children: [
                      Expanded(child: SizedBox()),
                      Center(child: GestureDetector(
                        onTap: () async{
                          final Uri url = Uri.parse('https://aibiblegames.com/contact/');
                          if (!await launchUrl(url)) {
                            throw Exception('Could not launch $url');
                          }
                        },
                        child: Stack(
                          children: [
                            SizedBox(
                              width: 100,
                              height: 50,
                              child: Image.asset(
                                'assets/bg_btn.png',
                                fit: BoxFit.contain,
                              ),
                            ),
                            SizedBox(
                              width: 100,
                              height: 50,
                              child: Center(
                                child: Text('CONTACT US', style: MyStyles.primaryButtonSmallStyle,),
                              ),
                            )
                          ],
                        ),
                      ),),
                      SizedBox(width: 20,),
                      Center(child: GestureDetector(
                        onTap: () async{
                          final Uri url = Uri.parse('https://aibiblegames.com/faq/');
                          if (!await launchUrl(url)) {
                            throw Exception('Could not launch $url');
                          }
                        },
                        child: Stack(
                          children: [
                            SizedBox(
                              width: 100,
                              height: 50,
                              child: Image.asset(
                                'assets/bg_btn.png',
                                fit: BoxFit.contain,
                              ),
                            ),
                            SizedBox(
                              width: 100,
                              height: 50,
                              child: Center(
                                child: Text('FAQ', style: MyStyles.primaryButtonSmallStyle,),
                              ),
                            )
                          ],
                        ),
                      ),),
                      Expanded(child: SizedBox()),
                    ],
                  )
                ],
              ),
            ),
          );
        },

      ),
      // floatingActionButton: FloatingActionButton.extended(
      //   onPressed: () async{
      //     DocumentSnapshot docSnap = await documentStream.first;
      //     dynamic activeSubscription = docSnap.data().toString().contains('subscription') == true ? docSnap.get('subscription') : null;
      //     if(activeSubscription == null){
      //       showSubscriptionDialog(context);
      //     }
      //     else{
      //       Navigator.of(context).pushNamed('/games');
      //     }
      //
      //   },
      //   icon: Icon(Icons.add),
      //   label: Text("New Game"),
      // ),

    );
  }
  void showSubscriptionDialog(BuildContext context) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: Text("Subscription"),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text(
                "You don't have a subscription plan. With the free plan, you can enjoy a fully immersive gaming experience. Would you like to subscribe?",
                textAlign: TextAlign.center,
              ),
              SizedBox(height: 20),
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: <Widget>[
                  ElevatedButton(
                    onPressed: () {
                      // Add your subscription handling logic here.
                      // Typically, you would initiate a payment process.
                      Navigator.of(context).pop(); // Close the dialog.
                      // You can navigate to a payment screen or perform other actions.
                      Navigator.of(context).pushNamed("/subscriptions");
                    },
                    child: Text("Yes"),
                  ),
                  ElevatedButton(
                    onPressed: () {
                      Navigator.of(context).pop(); // Close the dialog.
                      // Add logic for playing the free game here.
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (_) => ChatScreen(gameMode: 'free',userName: userName,),
                        ),
                      );
                    },
                    child: Text("No, Play free game"),
                  ),
                ],
              ),
            ],
          ),
        );
      },
    );
  }
}