import 'package:aibiblegames/pages/ChatScreen.dart';
import 'package:aibiblegames/pages/SubscriptionsScreen.dart';
import 'package:aibiblegames/utils/MyColors.dart';
import 'package:aibiblegames/utils/MyStyles.dart';
import 'package:carousel_slider/carousel_slider.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:loader_overlay/loader_overlay.dart';
import 'package:url_launcher/url_launcher.dart';

import '../utils/CommonUtils.dart';

class SplashScreen extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return SplashScreenState();
  }
}

class SplashScreenState extends State<SplashScreen> {
  TextEditingController userNameController = TextEditingController();
  TextEditingController emailController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  TextEditingController confirmPasswordController = TextEditingController();
  TextEditingController fullNameController = TextEditingController();
  bool isPassSecure = true, isSignUpPassSecure = true, isSignUpConfirmPassSecure = true;
  int pageMode = 0; // 1: Play Free Game, 2: Login, 3: Signup
  @override
  void initState() {
    super.initState();
    FirebaseAuth.instance
        .authStateChanges()
        .listen((User? user) {
      if (user != null && user.emailVerified) {
        print('Monitoring Auth');
        Navigator.pushReplacementNamed(context,'/home');
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        padding: EdgeInsets.all(20),
        height: MediaQuery.of(context).size.height,
        width: MediaQuery.of(context).size.width,
        decoration: BoxDecoration(
            color: Colors.black87,
            image: DecorationImage(
                image: AssetImage("assets/images/background/bg1.png"),
                fit: BoxFit.cover)),
        child: Column(
          children: [
            Expanded(child: SingleChildScrollView(
              child: ListBody(
                children: [
                  SizedBox(height: 10),
                  Row(
                    children: [
                      pageMode != 0 ? GestureDetector(
                        onTap: () {
                          setState(() {
                            pageMode = 0;
                          });
                        },
                        child: Stack(
                          children: [
                            SizedBox(
                              width: 50,
                              height: 50,
                              child: Image.asset(
                                'assets/bg_btn_back.png',
                                fit: BoxFit.contain,
                              ),
                            )
                          ],
                        ),
                      ) : SizedBox(),
                      Expanded(child: SizedBox()),
                      GestureDetector(
                        onTap: () async{
                          final Uri url = Uri.parse('https://aibiblegames.com/instructions/');
                          if (!await launchUrl(url)) {
                            throw Exception('Could not launch $url');
                          }
                        },
                        child: Stack(
                          children: [
                            SizedBox(
                              width: 50,
                              height: 50,
                              child: Image.asset(
                                'assets/bg_circle_button.png',
                                fit: BoxFit.contain,
                              ),
                            ),
                            SizedBox(
                              width: 50,
                              height: 50,
                              child: Center(
                                child: Icon(Icons.list, size: 35,color: MyColors.iconCOlor,),
                              ),
                            )
                          ],
                        ),
                      ),
                      SizedBox(width: 10,),
                      GestureDetector(
                        onTap: () async{
                          final Uri url = Uri.parse('https://aibiblegames.com/demo.mp4');
                          if (!await launchUrl(url)) {
                            throw Exception('Could not launch $url');
                          }
                        },
                        child: Stack(
                          children: [
                            SizedBox(
                              width: 50,
                              height: 50,
                              child: Image.asset(
                                'assets/bg_circle_button.png',
                                fit: BoxFit.contain,
                              ),
                            ),
                            SizedBox(
                              width: 50,
                              height: 50,
                              child: Center(
                                child: Icon(Icons.play_arrow, size: 35,color: MyColors.iconCOlor,),
                              ),
                            )
                          ],
                        ),
                      )
                    ],
                  ),

                  Text(
                    "AI BIBLE\nGAMES".toUpperCase(),
                    style: const TextStyle(
                        fontSize: 50, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7),
                    textAlign: TextAlign.center,
                  ),
                  pageMode != 3? CarouselSlider(
                    items: [1, 2, 3, 4, 5, 6].map((index) {
                      return Builder(builder: (BuildContext context) {
                        return Container(
                          padding: EdgeInsets.all(8), // Border width
                          child: ClipRRect(
                            child: Image.asset("assets/images/carousel/${index}.png", fit: BoxFit.cover),
                            borderRadius: BorderRadius.circular(30),
                          ),
                        );
                        // ClipRRect(
                        //   child: Image.asset("assets/images/carousel/${index}.png"),
                        //   borderRadius: BorderRadius.circular(20),
                        // );
                      });
                    }).toList(),
                    options: CarouselOptions(
                      enableInfiniteScroll: true,
                      autoPlay: true,
                      aspectRatio: 1.2,
                      enlargeCenterPage: true,
                    ),
                  ): SizedBox(),
                  SizedBox(height: 0),

                  pageMode == 0 ? Center(child: GestureDetector(
                    onTap: () async{
                      var result = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (_) => SubscriptionsScreen(
                              fromScreen: 'splash'
                          ),
                        ),
                      );
                      print('Result: ${result}');
                      if(result == 0){
                        setState(() {
                          pageMode = 1;
                        });
                      }
                      else if(result == 1){
                        CommonUtils.showToast("Please create an account to subscribe to AI Bible Games!");
                        setState(() {
                          pageMode = 3;
                        });
                      }

                    },
                    child: Stack(
                      children: [
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Image.asset(
                            'assets/bg_btn.png',
                            fit: BoxFit.contain,
                          ),
                        ),
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Center(
                            child: Text('PLAY FREE GAME', style: MyStyles.primaryButtonStyle,),
                          ),
                        )
                      ],
                    ),
                  ),): SizedBox(),
                  pageMode == 0 ? SizedBox(height: 5) : SizedBox(),
                  pageMode == 2 ? Padding(
                    //padding: const EdgeInsets.only(left:15.0,right: 15.0,top:0,bottom: 0),
                    padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                      controller: emailController,
                      style: TextStyle(color: Colors.white),
                      decoration: InputDecoration(
                        label: Text('Enter your email'),
                        labelStyle: TextStyle(color: Colors.white),
                        enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                        focusedBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                      ),
                    ),
                  ): SizedBox(),
                  pageMode == 2 ? Padding(
                    padding: const EdgeInsets.only(
                        left: 15.0, right: 15.0, top: 15, bottom: 0),
                    //padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                      controller: passwordController,
                      obscureText: isPassSecure,
                      style: TextStyle(color: Colors.white),
                      decoration: InputDecoration(
                          label: Text('Enter your password'),
                          labelStyle: TextStyle(color: Colors.white),
                          enabledBorder: OutlineInputBorder(
                              borderSide: BorderSide(
                                  color: Colors.white
                              )
                          ),
                          focusedBorder: OutlineInputBorder(
                              borderSide: BorderSide(
                                  color: Colors.white
                              )
                          ),
                          suffixIcon: InkWell(
                            child: Icon(Icons.remove_red_eye, color: Colors.white,),
                            onTap: (){
                              setState(() {
                                isPassSecure = !isPassSecure;
                              });
                            },
                          )
                      ),

                    ),
                  ): SizedBox(),
                  pageMode == 3 ? Padding(
                    padding: const EdgeInsets.only(
                        left: 15.0, right: 15.0, top: 15, bottom: 15),
                    //padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                      controller: fullNameController,
                      style: TextStyle(color: Colors.white),
                      decoration: const InputDecoration(
                        label: Text('Enter your full name'),
                        labelStyle: TextStyle(color: Colors.white),
                        enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                        focusedBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                      ),
                    ),
                  ): SizedBox(),
                  pageMode == 3 ? Padding(
                    //padding: const EdgeInsets.only(left:15.0,right: 15.0,top:0,bottom: 0),
                    padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                      controller: emailController,
                      style: TextStyle(color: Colors.white),
                      decoration: const InputDecoration(
                        label: Text('Enter your email'),
                        labelStyle: TextStyle(color: Colors.white),
                        enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                        focusedBorder: OutlineInputBorder(
                            borderSide: BorderSide(
                                color: Colors.white
                            )
                        ),
                      ),
                    ),
                  ): SizedBox(),
                  pageMode == 3 ? Padding(
                    padding: const EdgeInsets.only(
                        left: 15.0, right: 15.0, top: 15, bottom: 0),
                    //padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                      controller: passwordController,
                      style: TextStyle(color: Colors.white),
                      obscureText: isSignUpPassSecure,
                      decoration: InputDecoration(
                          label: Text('Enter your password'),
                          labelStyle: TextStyle(color: Colors.white),
                          enabledBorder: OutlineInputBorder(
                              borderSide: BorderSide(
                                  color: Colors.white
                              )
                          ),
                          focusedBorder: OutlineInputBorder(
                              borderSide: BorderSide(
                                  color: Colors.white
                              )
                          ),
                          suffixIcon: InkWell(
                            child: Icon(Icons.remove_red_eye, color: Colors.white,),
                            onTap: (){
                              setState(() {
                                isSignUpPassSecure = !isSignUpPassSecure;
                              });
                            },
                          )
                      ),
                    ),
                  ): SizedBox(),
                  pageMode == 3 ? Padding(
                    padding: const EdgeInsets.only(
                        left: 15.0, right: 15.0, top: 15, bottom: 0),
                    //padding: EdgeInsets.symmetric(horizontal: 15),
                    child: TextField(
                        controller: confirmPasswordController,
                        style: TextStyle(color: Colors.white),
                        obscureText: isSignUpConfirmPassSecure,
                        decoration: InputDecoration(
                            label: Text('Confirm your password'),
                            labelStyle: TextStyle(color: Colors.white),
                            enabledBorder: OutlineInputBorder(
                                borderSide: BorderSide(
                                    color: Colors.white
                                )
                            ),
                            focusedBorder: OutlineInputBorder(
                                borderSide: BorderSide(
                                    color: Colors.white
                                )
                            ),
                            suffixIcon: InkWell(
                              child: Icon(Icons.remove_red_eye, color: Colors.white,),
                              onTap: (){
                                setState(() {
                                  isSignUpConfirmPassSecure = !isSignUpConfirmPassSecure;
                                });
                              },
                            )
                        )
                    ),
                  ): SizedBox(),
                  pageMode == 3 ? Padding(
                    padding: const EdgeInsets.only(
                        left: 15.0, right: 15.0, top: 20, bottom: 0),
                    //padding: EdgeInsets.symmetric(horizontal: 15),
                    child: Column(
                      children: [
                        Text('By continuing you agree to our', style: TextStyle(color: Colors.white),),
                        Row(
                          children: [
                            Expanded(child: SizedBox()),
                            InkWell(
                              onTap: ()async {
                                final Uri url = Uri.parse('https://aibiblegames.com/terms/');
                                if (!await launchUrl(url)) {
                                  throw Exception('Could not launch $url');
                                }
                              },
                              child: Text('"Terms of Service"', style: TextStyle(color: Colors.lightBlueAccent)),
                            ),
                            Text(' and ', style: TextStyle(color: Colors.white)),
                            InkWell(
                              onTap: ()async {
                                final Uri url = Uri.parse('https://aibiblegames.com/privacy/');
                                if (!await launchUrl(url)) {
                                  throw Exception('Could not launch $url');
                                }
                              },
                              child: Text('"Privacy Policy"', style: TextStyle(color: Colors.lightBlueAccent)),
                            ),
                            Expanded(child: SizedBox()),
                          ],
                        )


                      ],),
                  ): SizedBox(),
                  pageMode == 0 ? SizedBox(height: 5) : SizedBox(),
                  pageMode == 2 ? SizedBox(height: 10) : SizedBox(),
                  pageMode == 0 || pageMode == 2 ? Center(child: GestureDetector(
                    onTap: () async{
                      // Navigator.of(context).pushNamed('/login');
                      if(pageMode != 2){
                        setState(() {
                          pageMode = 2;
                        });
                      }
                      else{
                        if(emailController.text.isNotEmpty && passwordController.text.isNotEmpty){
                          context.loaderOverlay.show();
                          try {
                            UserCredential userCredential = await FirebaseAuth.instance.signInWithEmailAndPassword(
                              email: emailController.text,
                              password: passwordController.text,
                            );
                            if(userCredential.user?.emailVerified == false){
                              await FirebaseAuth.instance.signOut();
                              CommonUtils.showToast("Please verify your email.");
                            }
                            context.loaderOverlay.hide();
                          } on FirebaseAuthException catch (e) {
                            context.loaderOverlay.hide();
                            if (e.code == 'user-not-found') {
                              CommonUtils.showToast('No user found for that email.');
                            } else if (e.code == 'wrong-password') {
                              CommonUtils.showToast('Wrong password provided for that user.');
                            }
                            else{
                              CommonUtils.showToast(e.message ?? e.code);
                            }
                          }
                          catch(e){
                            context.loaderOverlay.hide();
                            CommonUtils.showToast(e.toString());
                          }
                        }
                        else{
                          CommonUtils.showToast("Please enter your email and password.");
                        }
                      }

                    },
                    child: Stack(
                      children: [
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Image.asset(
                            'assets/bg_btn.png',
                            fit: BoxFit.contain,
                          ),
                        ),
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Center(
                            child: Text('LOGIN', style: MyStyles.primaryButtonStyle,),
                          ),
                        )
                      ],
                    ),
                  ),) : SizedBox(),
                  pageMode == 0 ? SizedBox(height: 10): SizedBox(),
                  pageMode == 3 ? SizedBox(height: 20) : SizedBox(),
                  pageMode == 0  || pageMode == 3 ? Center(child: GestureDetector(
                    onTap: () async{
                      // Navigator.of(context).pushNamed('/signup');
                      if(pageMode != 3){
                        setState(() {
                          pageMode = 3;
                        });
                      }
                      else{
                        if(fullNameController.text.isEmpty){
                          CommonUtils.showToast("Please enter your full name!");
                          return;
                        }
                        if(emailController.text.isEmpty){
                          CommonUtils.showToast("Please enter your email!");
                          return;
                        }
                        if(passwordController.text.isEmpty){
                          CommonUtils.showToast("Please enter your password!");
                          return;
                        }
                        if(confirmPasswordController.text.isEmpty){
                          CommonUtils.showToast("Please enter your password confirmation!");
                          return;
                        }
                        if(confirmPasswordController.text != passwordController.text){
                          CommonUtils.showToast("Password is not match!");
                          return;
                        }
                        context.loaderOverlay.show();
                        try{
                          UserCredential userCredential = await FirebaseAuth.instance.createUserWithEmailAndPassword(
                              email: emailController.text,
                              password: passwordController.text
                          );
                          if(userCredential.user != null){
                            await userCredential.user?.updateDisplayName(fullNameController.text);
                            await userCredential.user?.sendEmailVerification();
                            await FirebaseAuth.instance.signOut();
                            context.loaderOverlay.hide();
                            CommonUtils.showToast("We sent confirmation email. Please activate your account.");
                            setState(() {
                              pageMode = 2;
                            });
                          }

                        } on FirebaseAuthException catch (e) {
                          context.loaderOverlay.hide();
                          if (e.code == 'weak-password') {
                            CommonUtils.showToast('The password provided is too weak.');
                          } else if (e.code == 'email-already-in-use') {
                            CommonUtils.showToast('The account already exists for that email.');
                          }
                          else{
                            CommonUtils.showToast(e.message ?? '');
                          }

                        } catch (e) {
                          context.loaderOverlay.hide();
                          CommonUtils.showToast(e.toString());
                        }
                      }

                    },
                    child: Stack(
                      children: [
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Image.asset(
                            'assets/bg_btn.png',
                            fit: BoxFit.contain,
                          ),
                        ),
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Center(
                            child: Text('SIGN UP', style: MyStyles.primaryButtonStyle,),
                          ),
                        )
                      ],
                    ),
                  ),): SizedBox(),
                  pageMode == 1 ? TextField(
                    controller: userNameController,

                    decoration: InputDecoration(
                      label: Text('Enter your name'),
                      labelStyle: TextStyle(color: Colors.white),
                      enabledBorder: OutlineInputBorder(
                          borderSide: BorderSide(
                              color: Colors.white
                          )
                      ),
                      focusedBorder: OutlineInputBorder(
                          borderSide: BorderSide(
                              color: Colors.white
                          )
                      ),
                    ),
                    style: TextStyle(color: Colors.white, fontSize: 20),
                  ): SizedBox(),
                  pageMode == 1 ? SizedBox(height: 15): SizedBox(),
                  pageMode == 1 ? Center(child: GestureDetector(
                    onTap: () {
                      if (userNameController.text.isNotEmpty) {
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (_) => ChatScreen(
                              gameMode: 'free',
                              userName: userNameController.text,
                            ),
                          ),
                        );
                      } else {}
                    },
                    child: Stack(
                      children: [
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Image.asset(
                            'assets/bg_btn.png',
                            fit: BoxFit.contain,
                          ),
                        ),
                        SizedBox(
                          width: 200,
                          height: 50,
                          child: Center(
                            child: Text('PLAY GAME', style: MyStyles.primaryButtonStyle,),
                          ),
                        )
                      ],
                    ),
                  ),) : SizedBox(),

                  SizedBox(height: 0),

                ],
              ),
            )),
            Row(
              children: [
                Expanded(child: SizedBox()),
                Center(child: GestureDetector(
                  onTap: () async{
                    final Uri url = Uri.parse('https://aibiblegames.com/contact/');
                    if (!await launchUrl(url)) {
                      throw Exception('Could not launch $url');
                    }
                  },
                  child: Stack(
                    children: [
                      SizedBox(
                        width: 100,
                        height: 50,
                        child: Image.asset(
                          'assets/bg_btn.png',
                          fit: BoxFit.contain,
                        ),
                      ),
                      SizedBox(
                        width: 100,
                        height: 50,
                        child: Center(
                          child: Text('CONTACT US', style: MyStyles.primaryButtonSmallStyle,),
                        ),
                      )
                    ],
                  ),
                ),),
                SizedBox(width: 20,),
                Center(child: GestureDetector(
                  onTap: () async{
                    final Uri url = Uri.parse('https://aibiblegames.com/faq/');
                    if (!await launchUrl(url)) {
                      throw Exception('Could not launch $url');
                    }
                  },
                  child: Stack(
                    children: [
                      SizedBox(
                        width: 100,
                        height: 50,
                        child: Image.asset(
                          'assets/bg_btn.png',
                          fit: BoxFit.contain,
                        ),
                      ),
                      SizedBox(
                        width: 100,
                        height: 50,
                        child: Center(
                          child: Text('FAQ', style: MyStyles.primaryButtonSmallStyle,),
                        ),
                      )
                    ],
                  ),
                ),),
                Expanded(child: SizedBox()),
              ],
            )
          ],
        ),
      )
    );
  }
}
