import 'dart:developer';

import 'package:aibiblegames/utils/CommonUtils.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:cloud_functions/cloud_functions.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter_stripe/flutter_stripe.dart';
import 'package:intl/intl.dart';
import 'package:loader_overlay/loader_overlay.dart';

import '../utils/MyColors.dart';
import '../utils/MyStyles.dart';

class SubscriptionsScreen extends StatefulWidget{
  String fromScreen = '';
  SubscriptionsScreen({String? fromScreen}){
    if(fromScreen != null){
      this.fromScreen = fromScreen;
    }
  }
  @override
  State<SubscriptionsScreen> createState() => _SubscriptionsScreenState();

}
FirebaseFirestore firestore = FirebaseFirestore.instance;
class _SubscriptionsScreenState extends State<SubscriptionsScreen>{
  List<dynamic> stripeSubscriptionPlans = [];
  String selectedSubscription = '';
  String selectedSubscriptionID = '';
  dynamic selectedSubscriptionObj;
  final Stream<DocumentSnapshot> documentStream = firestore.collection('users').doc(FirebaseAuth.instance.currentUser?.uid).snapshots();
  loadSubscriptionPlans()async{
    context.loaderOverlay.show();
    try{
      final response = await FirebaseFunctions.instance
          .httpsCallable('fetchStripeSubscriptionPlans')
          .call();
      print('Result : ${response.data['data']}');
      setState(() {
        stripeSubscriptionPlans = response.data['data'];
      });
      context.loaderOverlay.hide();
    }on FirebaseFunctionsException catch (error) {
      context.loaderOverlay.hide();
      CommonUtils.showToast(error.message ?? '');
    }
    catch(e){
      context.loaderOverlay.hide();
      print(e.toString());
      CommonUtils.showToast(e.toString() ?? '');
    }
  }
  @override
  void initState() {
    super.initState();
    loadSubscriptionPlans();
  }
  @override
  Widget build(BuildContext context) {
    return SafeArea(child: Scaffold(

      body: StreamBuilder<DocumentSnapshot>(
        stream: documentStream,
        builder: (context, snapshot){
          if (snapshot.hasError) {
            return Text('Something went wrong');
          }

          if (snapshot.connectionState == ConnectionState.waiting) {
            return Text("Loading");
          }
          print('snapshot.data: ${snapshot.data?.data().toString().contains('subscription')}');
          dynamic activeSubscription = snapshot.data?.data().toString().contains('subscription') == true ? snapshot.data!.get('subscription') : null;
          log(activeSubscription.toString());
          bool hasSubscription = activeSubscription != null && activeSubscription['status'] == 'active';
          dynamic activeSubscriptionPlan;
          if(hasSubscription){
            String activeSubscriptionPlanId = snapshot.data!.get('subscription')['plan']['id'];

            stripeSubscriptionPlans.forEach((element) {
              if(element['default_price'] == activeSubscriptionPlanId){
                activeSubscriptionPlan = element;
              }
            });
          }
         return Container(
           decoration: BoxDecoration(
               color: Colors.black87,
               image: DecorationImage(
                   image: AssetImage("assets/images/background/bg2.png"),
                   fit: BoxFit.cover)),
           child: Stack(
             children: [
               // Button in the top-left corner
               Align(
                 alignment: Alignment.topLeft,
                 child: Padding(
                   padding: EdgeInsets.all(16.0),
                   child: Row(
                     children: [
                       GestureDetector(
                         onTap: () {
                           Navigator.of(context).pop(-1);
                         },
                         child: Stack(
                           children: [
                             SizedBox(
                               width: 50,
                               height: 50,
                               child: Image.asset(
                                 'assets/bg_btn_back.png',
                                 fit: BoxFit.contain,
                               ),
                             )
                           ],
                         ),
                       ),
                       SizedBox(width: 20,),
                       GestureDetector(
                         onTap: () {
                           Navigator.of(context).pop(0);
                         },
                         child: Stack(
                           children: [
                             SizedBox(
                               width: 100,
                               height: 50,
                               child: Image.asset(
                                 'assets/bg_btn.png',
                                 fit: BoxFit.contain,
                               ),
                             ),
                             SizedBox(
                               width: 100,
                               height: 50,
                               child: Center(
                                 child: Text('PLAY GAME', style: MyStyles.primaryButtonSmallStyle,),
                               ),
                             )
                           ],
                         ),
                       )
                     ],
                   ),
                 ),
               ),
               // Text at the bottom center
               Center(
                 child: Align(
                   alignment: Alignment.bottomCenter,
                   child: Container(
                     padding: EdgeInsets.all(10),
                     child: Column(
                       mainAxisSize: MainAxisSize.min,
                       children: <Widget>[
                         Container(
                           child: Column(
                             children: [
                               Row(children: [
                                 Expanded(child: SizedBox()),
                                 Image.asset("assets/img_ad_free.png"),
                                 Text('AND', style: const TextStyle(
                                     fontSize: 40, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                                 Expanded(child: SizedBox()),
                               ],),
                               Text('GET ACCESS TO ADULT AND CHILDREN\'S\nVERSIONS OF:"SCRIPTURE SESSIONS"', style: const TextStyle(
                                   fontSize: 30, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                               Text('GET ACCESS TO: "NEW TESTAMENT: THE AI EXPERIENCE"', style: const TextStyle(
                                   fontSize: 30, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                               Text('GET ACCESS TO: "NEW TESTAMENT: TRIVIA CHALLENGE"', style: const TextStyle(
                                   fontSize: 30, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                               ListView.separated(
                                 shrinkWrap: true,
                                 physics: NeverScrollableScrollPhysics(),
                                 itemCount: stripeSubscriptionPlans.length,
                                 separatorBuilder: (BuildContext context, int index) => Divider(
                                   height: 3,
                                   color: Colors.transparent,
                                 ),
                                 itemBuilder: (context, index) {
                                   bool isCurrentPlanActive = hasSubscription && activeSubscriptionPlan['name'] == stripeSubscriptionPlans[index]['name'];
                                   return InkWell(
                                     onTap: !isCurrentPlanActive ? (){
                                       setState(() {
                                         selectedSubscription = stripeSubscriptionPlans[index]['default_price'];
                                         selectedSubscriptionObj = stripeSubscriptionPlans[index];
                                       });
                                     } : null,
                                     child: Container(
                                       padding: EdgeInsets.all(5),
                                       child: Container(
                                           decoration: BoxDecoration(
                                               color: isCurrentPlanActive ? Colors.green.shade700 : selectedSubscription == stripeSubscriptionPlans[index]['default_price'] ? Color(0xFFFFBF00): Color(0xFFEBE0D4),
                                               borderRadius: BorderRadius.all(Radius.circular(100))
                                           ),
                                           padding: const EdgeInsets.only(left: 20, right: 20, top: 10, bottom: 10),
                                           child: Row(
                                             children: [
                                               Text('${index + 1}', style: const TextStyle(
                                                   fontSize: 80, color: Colors.black87, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                                               SizedBox(width: 20,),
                                               Expanded(child: Column(
                                                 children: [
                                                   Row(
                                                     children: [
                                                       Expanded(child:Column(children:[

                                                         Row(children: [Text(stripeSubscriptionPlans[index]['name'], style: const TextStyle(
                                                             fontSize: 40, color: Colors.black87, fontFamily: 'Jomhuria', height: 0.7))],),
                                                         Row(children: [Text('SUBSCRIPTION', style: const TextStyle(
                                                             fontSize: 30, color: Colors.black87, fontFamily: 'Jomhuria', height: 0.7))],)
                                                       ])),
                                                       Column(
                                                         children: [
                                                           Align(
                                                             alignment: Alignment.centerRight,
                                                             child:Text('\$${stripeSubscriptionPlans[index]['price']['unit_amount'] / 100}', style: const TextStyle(
                                                                   fontSize: 40, color: Colors.black87, fontFamily: 'Jomhuria', height: 0.7))
                                                           ),
                                                           isCurrentPlanActive ? Container(
                                                             child: Text('Renewed on\n${DateFormat('dd/MM/yyyy').format(DateTime.fromMillisecondsSinceEpoch(activeSubscription['current_period_end'] * 1000))}', style: const TextStyle(
                                                                 fontSize: 20, color: Colors.black87, fontFamily: 'Jomhuria', height: 0.7)),
                                                           ): SizedBox()
                                                         ],
                                                       )

                                                       // Radio(value: stripeSubscriptionPlans[index]['id'], groupValue: selectedSubscription, onChanged: (value){setState(() {
                                                       //   selectedSubscription = stripeSubscriptionPlans[index]['id'];
                                                       // });})
                                                     ],
                                                   ),

                                                 ],
                                               ))
                                             ],
                                           )
                                       ),
                                     ),
                                   );
                                 },
                               ),
                               Center(child: GestureDetector(
                                 onTap: () async{
                                    if(widget.fromScreen == "splash"){
                                      Navigator.of(context).pop(1);
                                    }
                                    else{

                                      if(selectedSubscription == ""){
                                        CommonUtils.showToast("Please choose your subscription plan");
                                      }
                                      else if (hasSubscription && selectedSubscription == activeSubscription['default_price']){
                                        CommonUtils.showToast("You have already current plan. Please choose another plan to update.");
                                      }
                                      else {
                                        context.loaderOverlay.show();
                                        try{
                                          print(selectedSubscriptionObj);
                                          final response = await FirebaseFunctions.instance
                                              .httpsCallable('createPaymentIntent')
                                              .call(<String, dynamic > {
                                            'selectedSubscription': selectedSubscription,
                                            'mode' : hasSubscription ? 'update' : 'create',
                                            'subscriptionId': hasSubscription ? activeSubscription['id'] : '',
                                            'newSubscriptionID': hasSubscription? selectedSubscriptionID : '',
                                            'subItemId': hasSubscription ? activeSubscription['items']['data'][0]['id'] : ''
                                          });

                                          String currentUserName = FirebaseAuth.instance.currentUser?.displayName ?? '';

                                          await Stripe.instance.initPaymentSheet(
                                            paymentSheetParameters: SetupPaymentSheetParameters(
                                              paymentIntentClientSecret: response.data['clientSecret'], //Gotten from payment intent
                                              style: ThemeMode.light,
                                              googlePay: const PaymentSheetGooglePay(merchantCountryCode: "US", currencyCode: "USD", testEnv: true),
                                              merchantDisplayName: currentUserName,
                                              customerId: response.data['clientSecret'],
                                              allowsDelayedPaymentMethods: true,



                                            ),
                                          );

                                          displayPaymentSheet(response.data['clientSecret']);


                                          context.loaderOverlay.hide();

                                        }
                                        catch (e) {
                                          context.loaderOverlay.hide();
                                          print(e.toString());
                                        }
                                      }
                                    }
                                 },
                                 child: Stack(
                                   children: [
                                     SizedBox(
                                       width: 200,
                                       height: 50,
                                       child: Image.asset(
                                         'assets/bg_btn.png',
                                         fit: BoxFit.contain,
                                       ),
                                     ),
                                     SizedBox(
                                       width: 200,
                                       height: 50,
                                       child: Center(
                                         child: Text(hasSubscription ? 'UPDATE' : 'CONTINUE', style: MyStyles.primaryButtonStyle,),
                                       ),
                                     )
                                   ],
                                 ),
                               ),),
                                SizedBox(height: 5,),
                               hasSubscription ? Center(child: GestureDetector(
                                 onTap: () async{
                                   if(widget.fromScreen == "splash"){
                                     Navigator.of(context).pop(1);
                                   }
                                   else{


                                       context.loaderOverlay.show();
                                       try{
                                         final response = await FirebaseFunctions.instance
                                             .httpsCallable('cancelSubscription')
                                             .call(<String, dynamic > {
                                           'subscriptionId': activeSubscription['id']
                                         });
                                         CommonUtils.showToast('Your subscription is canceled.');

                                         context.loaderOverlay.hide();

                                       }
                                       catch (e) {
                                         context.loaderOverlay.hide();
                                         print(e.toString());
                                       }

                                   }
                                 },
                                 child: Stack(
                                   children: [
                                     SizedBox(
                                       width: 200,
                                       height: 50,
                                       child: Image.asset(
                                         'assets/bg_btn_inactive.png',
                                         fit: BoxFit.contain,
                                       ),
                                     ),
                                     SizedBox(
                                       width: 200,
                                       height: 50,
                                       child: Center(
                                         child: Text('CANCEL', style: MyStyles.primaryButtonStyle,),
                                       ),
                                     )
                                   ],
                                 ),
                               ),) : SizedBox()
                             ],
                           ),
                         ),

                         // hasSubscription ? Container(
                         //   padding: EdgeInsets.all(20),
                         //   child: Column(
                         //     children: [
                         //       Text('You have subscribe to the ${activeSubscriptionPlan?['name']} plan.', style: const TextStyle(
                         //           fontSize: 40, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                         //       activeSubscriptionPlan != null ? Text('Your subscription is \$${(activeSubscriptionPlan?['price']['unit_amount'] ?? 0 ).toDouble()/100} per term.', style: const TextStyle(
                         //           fontSize: 40, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,) :  SizedBox(),
                         //       Text('You plan will be renewed on ${ DateFormat('dd/MM/yyyy').format(DateTime.fromMillisecondsSinceEpoch(activeSubscription['current_period_end'] * 1000))}.', style: const TextStyle(
                         //           fontSize: 40, color: Colors.white, fontFamily: 'Jomhuria', height: 0.7), textAlign: TextAlign.center,),
                         //       Center(child: GestureDetector(
                         //         onTap: () async{
                         //           if(widget.fromScreen == "splash"){
                         //             Navigator.of(context).pop(1);
                         //           }
                         //           else{
                         //
                         //             if(selectedSubscription == ""){
                         //               CommonUtils.showToast("Please choose your subscription plan");
                         //             }
                         //             else {
                         //               context.loaderOverlay.show();
                         //               try{
                         //                 final response = await FirebaseFunctions.instance
                         //                     .httpsCallable('createPaymentIntent')
                         //                     .call(<String, dynamic > {
                         //                   'selectedSubscription': selectedSubscription
                         //                 });
                         //
                         //                 String currentUserName = FirebaseAuth.instance.currentUser?.displayName ?? '';
                         //
                         //                 await Stripe.instance.initPaymentSheet(
                         //                   paymentSheetParameters: SetupPaymentSheetParameters(
                         //                     paymentIntentClientSecret: response.data['clientSecret'], //Gotten from payment intent
                         //                     style: ThemeMode.light,
                         //                     googlePay: const PaymentSheetGooglePay(merchantCountryCode: "US", currencyCode: "USD", testEnv: true),
                         //                     merchantDisplayName: currentUserName,
                         //                     customerId: response.data['clientSecret'],
                         //                     allowsDelayedPaymentMethods: true,
                         //
                         //
                         //                   ),
                         //                 );
                         //
                         //                 displayPaymentSheet(response.data['clientSecret']);
                         //
                         //
                         //                 context.loaderOverlay.hide();
                         //
                         //               }
                         //               catch (e) {
                         //                 context.loaderOverlay.hide();
                         //                 print(e.toString());
                         //               }
                         //             }
                         //           }
                         //         },
                         //         child: Stack(
                         //           children: [
                         //             SizedBox(
                         //               width: 200,
                         //               height: 50,
                         //               child: Image.asset(
                         //                 'assets/bg_btn.png',
                         //                 fit: BoxFit.contain,
                         //               ),
                         //             ),
                         //             SizedBox(
                         //               width: 200,
                         //               height: 50,
                         //               child: Center(
                         //                 child: Text('CONTINUE', style: MyStyles.primaryButtonStyle,),
                         //               ),
                         //             )
                         //           ],
                         //         ),
                         //       ),),
                         //     ],
                         //   ),
                         // ): SizedBox(),

                       ],
                     ),
                   ),
                 ),
               ),
             ],
           ),

         );
        },
      ),
    ));
  }
  displayPaymentSheet(String clientSecret) async {
    ////print("DISPLAY PAYMENT SHEET");
    try {
      await Stripe.instance.presentPaymentSheet().then((value) {
       print('Success');
       context.loaderOverlay.hide();
       Navigator.of(context).pop(1);
      }).onError((error, stackTrace) {
        throw Exception(error);
      });
    } on StripeException {
      context.loaderOverlay.hide();
      print('Catch 1');
    } catch (e) {
      context.loaderOverlay.hide();

      print('Catch 2');
    }
  }


}