import 'package:aibiblegames/utils/CommonUtils.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:loader_overlay/loader_overlay.dart';

class SignUpScreen extends StatefulWidget{
  @override
  State<SignUpScreen> createState() => _SignUpScreenState();

}

class _SignUpScreenState extends State<SignUpScreen>{
  TextEditingController emailController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  TextEditingController confirmPasswordController = TextEditingController();
  TextEditingController fullNameController = TextEditingController();
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      body: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.only(top: 60.0),
              child: Center(
                child: Container(
                    width: 200,
                    height: 150,
                    /*decoration: BoxDecoration(
                        color: Colors.red,
                        borderRadius: BorderRadius.circular(50.0)),*/
                    child: Image.asset('assets/logo.png')),
              ),
            ),
            SizedBox(height: 30,),
            Padding(
              padding: const EdgeInsets.only(
                  left: 15.0, right: 15.0, top: 15, bottom: 15),
              //padding: EdgeInsets.symmetric(horizontal: 15),
              child: TextField(
                controller: fullNameController,
                decoration: const InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'Full Name',
                    hintText: 'Enter your full name'),
              ),
            ),
            Padding(
              //padding: const EdgeInsets.only(left:15.0,right: 15.0,top:0,bottom: 0),
              padding: EdgeInsets.symmetric(horizontal: 15),
              child: TextField(
                controller: emailController,
                decoration: const InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'Email',
                    hintText: 'Enter valid email'),
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(
                  left: 15.0, right: 15.0, top: 15, bottom: 0),
              //padding: EdgeInsets.symmetric(horizontal: 15),
              child: TextField(
                controller: passwordController,
                obscureText: true,
                decoration: InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'Password',
                    hintText: 'Enter secure password'),
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(
                  left: 15.0, right: 15.0, top: 15, bottom: 0),
              //padding: EdgeInsets.symmetric(horizontal: 15),
              child: TextField(
                controller: confirmPasswordController,
                obscureText: true,
                decoration: InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'Confirm Password',
                    hintText: 'Confirm your password'),
              ),
            ),
            SizedBox(height: 30,),

            Container(
              height: 50,
              width: 250,
              decoration: BoxDecoration(
                  color: Colors.blue, borderRadius: BorderRadius.circular(20)),
              child: ElevatedButton(
                onPressed: () async{
                  if(fullNameController.text.isEmpty){
                    CommonUtils.showToast("Please enter your full name!");
                    return;
                  }
                  if(emailController.text.isEmpty){
                    CommonUtils.showToast("Please enter your email!");
                    return;
                  }
                  if(passwordController.text.isEmpty){
                    CommonUtils.showToast("Please enter your password!");
                    return;
                  }
                  if(confirmPasswordController.text.isEmpty){
                    CommonUtils.showToast("Please enter your password confirmation!");
                    return;
                  }
                  if(confirmPasswordController.text != passwordController.text){
                    CommonUtils.showToast("Password is not match!");
                    return;
                  }
                  context.loaderOverlay.show();
                  try{
                    UserCredential userCredential = await FirebaseAuth.instance.createUserWithEmailAndPassword(
                        email: emailController.text,
                        password: passwordController.text
                    );
                    if(userCredential.user != null){
                      await userCredential.user?.updateDisplayName(fullNameController.text);
                      await userCredential.user?.sendEmailVerification();
                      context.loaderOverlay.hide();
                      CommonUtils.showToast("We sent confirmation email. Please activate your account.");
                      Navigator.pushReplacementNamed(context, '/login');
                    }

                  } on FirebaseAuthException catch (e) {
                    context.loaderOverlay.hide();
                    if (e.code == 'weak-password') {
                      CommonUtils.showToast('The password provided is too weak.');
                    } else if (e.code == 'email-already-in-use') {
                      CommonUtils.showToast('The account already exists for that email.');
                    }
                    else{
                      CommonUtils.showToast(e.message ?? '');
                    }

                  } catch (e) {
                    context.loaderOverlay.hide();
                    CommonUtils.showToast(e.toString());
                  }

                },
                child: Text(
                  'SignUp',
                  style: TextStyle(color: Colors.white, fontSize: 25),
                ),
              ),
            ),
            SizedBox(
              height: 130,
            ),
            Row(
              children: [
                Expanded(child: SizedBox()),
                Text('Do you have already an account?'),
                InkWell(child: Text(' Login'), onTap: (){
                  Navigator.pushReplacementNamed(context, '/login');
                },),
                Expanded(child: SizedBox()),
              ],
            )
          ],
        ),
      ),
    );
  }

}